@@ -32,7 +32,6 @@ import (
 	"github.com/openshift/cloud-credential-operator/pkg/azure"
 	azuremock "github.com/openshift/cloud-credential-operator/pkg/azure/mock"
 	"github.com/openshift/cloud-credential-operator/pkg/operator/constants"
-
 	log "github.com/sirupsen/logrus"
 	"github.com/stretchr/testify/assert"
 	corev1 "k8s.io/api/core/v1"
@@ -61,8 +60,8 @@ const (
 var (
 	rootSecretMintAnnotation = corev1.Secret{
 		ObjectMeta: metav1.ObjectMeta{
-			Name:      azure.RootSecretName,
-			Namespace: azure.RootSecretNamespace,
+			Name:      constants.AzureCloudCredSecretName,
+			Namespace: constants.CloudCredSecretNamespace,
 			Annotations: map[string]string{
 				constants.AnnotationKey: constants.MintAnnotation,
 			},
@@ -113,7 +112,7 @@ func TestAnnotations(t *testing.T) {
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
 			f := fake.NewFakeClient(&tt.in, &validSecret)
-			actuator, err := azure.NewActuator(f)
+			actuator, err := azure.NewActuator(f, openshiftapiv1.AzurePublicCloud)
 			if err != nil {
 				assert.Regexp(t, tt.errRegexp, err)
 				assert.Nil(t, actuator)
