@@ -31,6 +31,7 @@ import (
 	"github.com/openshift/cloud-credential-operator/pkg/operator/internalcontroller"
 	"github.com/openshift/cloud-credential-operator/pkg/operator/metrics"
 	"github.com/openshift/cloud-credential-operator/pkg/operator/utils"
+	"github.com/openshift/cloud-credential-operator/pkg/util/clusteroperator"
 
 	configv1 "github.com/openshift/api/config/v1"
 	operatorv1 "github.com/openshift/api/operator/v1"
@@ -69,7 +70,8 @@ const (
 
 	credentialsRequestInfraMismatch = "InfrastructureMismatch"
 
-	// Hack for 4.5 to get us a status update when we don't have a cred request to reconcile.
+	// Hack: send a fake CredentialsRequest to get us a status update when we don't have one to reconcile.
+	// Should be removed when status update is broken out into it's own controller.
 	fakeCredRequestName     = "no-op-fake-cred-request"
 	fakeCredReuestNamespace = "no-op-fake-cred-request-namespace"
 )
@@ -83,11 +85,13 @@ func AddWithActuator(mgr manager.Manager, actuator actuator.Actuator, platType c
 
 // newReconciler returns a new reconcile.Reconciler
 func newReconciler(mgr manager.Manager, actuator actuator.Actuator, platType configv1.PlatformType) reconcile.Reconciler {
-	return &ReconcileCredentialsRequest{
+	r := &ReconcileCredentialsRequest{
 		Client:       mgr.GetClient(),
 		Actuator:     actuator,
 		platformType: platType,
 	}
+	clusteroperator.AddStatusHandler(r)
+	return r
 }
 
 // isFutureCredRequestSecret is used to identify if a given secret namespace + name is one we're expecting in a future
@@ -146,10 +150,10 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 	// We use an annotation on secrets that refers back to their owning credentials request because
 	// the normal owner reference is not namespaced, and we want to support credentials requests being
 	// in a centralized namespace, but writing secrets into component namespaces.
-	mapFn := handler.ToRequestsFunc(
+	targetCredSecretMapFunc := handler.ToRequestsFunc(
 		func(a handler.MapObject) []reconcile.Request {
 
-			// Hack for 4.5, reconcile a fake cred request we're watching for in the Reconcile loop, intended to be
+			// Hack: reconcile a fake cred request we're watching for in the Reconcile loop, intended to be
 			// a no-op so we can get a status update. (remove once we break status out into it's own controller)
 			if isFutureCredRequestSecret(actuator, a.Meta.GetNamespace(), a.Meta.GetName()) {
 				return []reconcile.Request{
@@ -222,13 +226,57 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 	err = c.Watch(
 		&source.Kind{Type: &corev1.Secret{}},
 		&handler.EnqueueRequestsFromMapFunc{
-			ToRequests: mapFn,
+			ToRequests: targetCredSecretMapFunc,
 		},
 		p)
 	if err != nil {
 		return err
 	}
 
+	// allCredRequestsMapFn simply looks up all CredentialsRequests and requests they be reconciled.
+	allCredRequestsMapFn := handler.ToRequestsFunc(
+		func(a handler.MapObject) []reconcile.Request {
+			log.Info("requeueing all CredentialsRequests")
+			crs := &minterv1.CredentialsRequestList{}
+			err := mgr.GetClient().List(context.TODO(), crs)
+			var requests []reconcile.Request
+			if err != nil {
+				log.WithError(err).Error("error listing all cred requests for requeue")
+				return requests
+			}
+			for _, cr := range crs.Items {
+				requests = append(requests, reconcile.Request{
+					NamespacedName: types.NamespacedName{
+						Name:      cr.Name,
+						Namespace: cr.Namespace,
+					},
+				})
+			}
+			return requests
+		})
+
+	adminCredSecretPredicate := predicate.Funcs{
+		UpdateFunc: func(e event.UpdateEvent) bool {
+			return isAdminCredSecret(e.MetaNew.GetNamespace(), e.MetaNew.GetName())
+		},
+		CreateFunc: func(e event.CreateEvent) bool {
+			return isAdminCredSecret(e.Meta.GetNamespace(), e.Meta.GetName())
+		},
+		DeleteFunc: func(e event.DeleteEvent) bool {
+			return isAdminCredSecret(e.Meta.GetNamespace(), e.Meta.GetName())
+		},
+	}
+	// Watch Secrets and reconcile if we see an event for an admin credential secret in kube-system.
+	err = c.Watch(
+		&source.Kind{Type: &corev1.Secret{}},
+		&handler.EnqueueRequestsFromMapFunc{
+			ToRequests: allCredRequestsMapFn,
+		},
+		adminCredSecretPredicate)
+	if err != nil {
+		return err
+	}
+
 	// Monitor namespace creation, and check out list of credentials requests for any destined
 	// for that new namespace. This allows us to be up and running for other components that don't
 	// yet exist, but will.
@@ -284,22 +332,27 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 		return err
 	}
 
-	// Mapper to reconcile all CredRequests any time something we're watching changes.
-	allCredRequestsMapFn := handler.ToRequestsFunc(
-		func(a handler.MapObject) []reconcile.Request {
-			crs := &minterv1.CredentialsRequestList{}
-			mgr.GetClient().List(context.TODO(), crs)
-			requests := []reconcile.Request{}
-			for _, cr := range crs.Items {
-				requests = append(requests, reconcile.Request{
-					NamespacedName: types.NamespacedName{
-						Name:      cr.Name,
-						Namespace: cr.Namespace,
-					},
-				})
-			}
-			return requests
-		})
+	// Monitor the cloud-credential-operator-config configmap to check if operator is re-enabled.
+	// Check if attempts were made to delete the credentials requests when the operator was disabled.
+	// We have to have to reconcile all those credential requests.
+	// Check if operator is re-enabled in cloud-credential-operator-config configmap
+	configMapPredicate := predicate.Funcs{
+		UpdateFunc: func(e event.UpdateEvent) bool {
+			return isCloudCredOperatorConfigMap(e.MetaNew)
+		},
+		CreateFunc: func(e event.CreateEvent) bool {
+			return isCloudCredOperatorConfigMap(e.Meta)
+		},
+		DeleteFunc: func(e event.DeleteEvent) bool {
+			return isCloudCredOperatorConfigMap(e.Meta)
+		},
+	}
+	err = c.Watch(
+		&source.Kind{Type: &corev1.ConfigMap{}},
+		&handler.EnqueueRequestsFromMapFunc{
+			ToRequests: allCredRequestsMapFn,
+		},
+		configMapPredicate)
 
 	// Watch the CloudCredential config object and reconcile everything on changes.
 	err = c.Watch(
@@ -311,31 +364,29 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 		return err
 	}
 
-	adminCredSecretPredicate := predicate.Funcs{
-		UpdateFunc: func(e event.UpdateEvent) bool {
-			return isAdminCredSecret(e.MetaNew.GetNamespace(), e.MetaNew.GetName())
-		},
-		CreateFunc: func(e event.CreateEvent) bool {
-			return isAdminCredSecret(e.Meta.GetNamespace(), e.Meta.GetName())
-		},
-		DeleteFunc: func(e event.DeleteEvent) bool {
-			return isAdminCredSecret(e.Meta.GetNamespace(), e.Meta.GetName())
-		},
-	}
-	// Watch Secrets and reconcile if we see an event for an admin credential secret in kube-system.
-	err = c.Watch(
-		&source.Kind{Type: &corev1.Secret{}},
-		&handler.EnqueueRequestsFromMapFunc{
-			ToRequests: allCredRequestsMapFn,
-		},
-		adminCredSecretPredicate)
-	if err != nil {
-		return err
-	}
-
 	return nil
 }
 
+// isCloudCredOperatorConfigMap returns true if given configmap is cloud-credential-operator-config configmap
+func isCloudCredOperatorConfigMap(cm metav1.Object) bool {
+	return cm.GetName() == constants.CloudCredOperatorConfigMap && cm.GetNamespace() == minterv1.CloudCredOperatorNamespace
+}
+
+func isAdminCredSecret(namespace, secretName string) bool {
+	if namespace == constants.CloudCredSecretNamespace {
+		if secretName == constants.AWSCloudCredSecretName ||
+			secretName == constants.AzureCloudCredSecretName ||
+			secretName == constants.GCPCloudCredSecretName ||
+			secretName == constants.OpenStackCloudCredsSecretName ||
+			secretName == constants.OvirtCloudCredsSecretName ||
+			secretName == constants.VSphereCloudCredSecretName {
+			log.WithField("secret", secretName).WithField("namespace", namespace).Info("observed admin cloud credential secret event")
+			return true
+		}
+	}
+	return false
+}
+
 var _ reconcile.Reconciler = &ReconcileCredentialsRequest{}
 
 // ReconcileCredentialsRequest reconciles a CredentialsRequest object
@@ -374,7 +425,7 @@ func (r *ReconcileCredentialsRequest) Reconcile(request reconcile.Request) (reco
 		metrics.MetricControllerReconcileTime.WithLabelValues(controllerName).Observe(dur.Seconds())
 	}()
 
-	// Hack for 4.5, reconcile a fake cred request we're watching for in the Reconcile loop, intended to be
+	// Hack: reconcile a fake cred request we're watching for in the Reconcile loop, intended to be
 	// a no-op so we can get a status update. (remove once we break status out into it's own controller)
 	if request.NamespacedName.Namespace == fakeCredReuestNamespace && request.NamespacedName.Name == fakeCredRequestName {
 		logger.Info("detected reconcile of fake credentials request name for status update purposes, returning")
@@ -706,7 +757,7 @@ func setIgnoredCondition(cr *minterv1.CredentialsRequest, clusterPlatform config
 
 	// Also clear any other conditions since we are ignoring this cred request,
 	// and we don't want to be in a degraded state b/c of cred requests that we're ignoring.
-	for _, cond := range constants.FailureConditionTypes {
+	for _, cond := range minterv1.FailureConditionTypes {
 		cr.Status.Conditions = utils.SetCredentialsRequestCondition(cr.Status.Conditions, cond,
 			corev1.ConditionFalse, reason, msg, updateCheck)
 	}
@@ -789,7 +840,7 @@ func crInfraMatches(cr *minterv1.CredentialsRequest, clusterCloudPlatform config
 }
 
 func checkForFailureConditions(cr *minterv1.CredentialsRequest) bool {
-	for _, t := range constants.FailureConditionTypes {
+	for _, t := range minterv1.FailureConditionTypes {
 		failureCond := utils.FindCredentialsRequestCondition(cr.Status.Conditions, t)
 		if failureCond != nil && failureCond.Status == corev1.ConditionTrue {
 			return true
@@ -797,18 +848,3 @@ func checkForFailureConditions(cr *minterv1.CredentialsRequest) bool {
 	}
 	return false
 }
-
-func isAdminCredSecret(namespace, secretName string) bool {
-	if namespace == constants.CloudCredSecretNamespace {
-		if secretName == constants.AWSCloudCredSecretName ||
-			secretName == constants.AzureCloudCredSecretName ||
-			secretName == constants.GCPCloudCredSecretName ||
-			secretName == constants.OpenStackCloudCredsSecretName ||
-			secretName == constants.OvirtCloudCredsSecretName ||
-			secretName == constants.VSphereCloudCredSecretName {
-			log.WithField("secret", secretName).WithField("namespace", namespace).Info("observed admin cloud credential secret event")
-			return true
-		}
-	}
-	return false
-}
